import { BaseEvent } from '#application/base/baseEvent'
import Database from '#application/database'
import { CharacterHair } from '#entities/characterHair'
import characterHairRepository from '#repositories/characterHairRepository'

interface IPayload {}

export default class characterHairListEvent extends BaseEvent {
  public listen(): void {
    this.socket.on('character:hair:list', this.handleEvent.bind(this))
  }

  private async handleEvent(data: IPayload, callback: (response: CharacterHair[]) => void): Promise<void> {
    try {
      const items: CharacterHair[] = await characterHairRepository.getAllSelectable()
      await Database.getEntityManager().populate(items, ['sprite'])
      return callback(items)
    } catch (error) {
      this.logger.error('character:hair:list error', error)
      return callback([])
    }
  }
}
