import { UUID } from '#application/types'
import { BaseEvent } from '#application/base/baseEvent'
import MapObjectRepository from '#repositories/mapObjectRepository'

type Payload = {
  id: UUID
  name: string
  tags: string[]
  originX: number
  originY: number
  isAnimated: boolean
  frameRate: number
  frameWidth: number
  frameHeight: number
}

export default class MapObjectUpdateEvent extends BaseEvent {
  public listen(): void {
    this.socket.on('gm:mapObject:update', this.handleEvent.bind(this))
  }

  private async handleEvent(data: Payload, callback: (success: boolean) => void): Promise<void> {
    try {
      if (!(await this.isCharacterGM())) return

      const mapObject = await MapObjectRepository.getById(data.id)
      if (!mapObject) return callback(false)

      await mapObject
        .setName(data.name)
        .setTags(data.tags)
        .setOriginX(data.originX)
        .setOriginY(data.originY)
        .setIsAnimated(data.isAnimated)
        .setFrameRate(data.frameRate)
        .setFrameWidth(data.frameWidth)
        .setFrameHeight(data.frameHeight)
        .update()

      return callback(true)
    } catch (error) {
      console.error(error)
      return callback(false)
    }
  }
}
